Index: boost/range/end.hpp
===================================================================
--- boost/range/end.hpp	(revision 67285)
+++ boost/range/end.hpp	(working copy)
@@ -17,6 +17,54 @@
 
 #include <boost/range/config.hpp>
 
+#ifndef BOOST_NO_RANGE_BASED_FOR
+#include <boost/mpl/bool.hpp>
+#include <boost/utility/enable_if.hpp>
+#include <boost/range/iterator.hpp>
+
+namespace boost
+{
+
+using std::end;
+
+namespace range_detail
+{
+    template <typename T>
+    struct has_end_impl
+    {
+        typedef char true_type;
+        typedef char false_type[2];
+
+        template <typename S>
+        static true_type& check(S* s_ptr, decltype(s_ptr->end())* = 0);
+
+        template <typename S>
+        static false_type& check(...);
+
+        static const bool value = sizeof(check<T>(0)) == sizeof(true_type);
+    };
+
+    template <typename T>
+    struct has_end : boost::mpl::bool_<has_end_impl<T>::value> {};
+}
+
+template <typename Range>
+inline typename boost::range_iterator<Range>::type
+end(Range& rng, typename disable_if<range_detail::has_end<Range>>::type* = 0)
+{
+    return range_end(rng);
+}
+
+template <typename Range>
+inline typename boost::range_iterator<Range const>::type
+end(Range const& rng, typename disable_if<range_detail::has_end<Range>>::type* = 0)
+{
+    return range_end(rng);
+}
+
+}
+#else
+
 #ifdef BOOST_NO_FUNCTION_TEMPLATE_ORDERING
 #include <boost/range/detail/end.hpp>
 #else
@@ -127,5 +175,7 @@
     }
 }
 
+#endif // BOOST_NO_RANGE_BASED_FOR
+
 #endif
 
Index: boost/range/begin.hpp
===================================================================
--- boost/range/begin.hpp	(revision 67285)
+++ boost/range/begin.hpp	(working copy)
@@ -17,6 +17,54 @@
 
 #include <boost/range/config.hpp>
 
+#ifndef BOOST_NO_RANGE_BASED_FOR
+#include <boost/mpl/bool.hpp>
+#include <boost/utility/enable_if.hpp>
+#include <boost/range/iterator.hpp>
+
+namespace boost
+{
+
+using std::begin;
+
+namespace range_detail
+{
+    template <typename T>
+    struct has_begin_impl
+    {
+        typedef char true_type;
+        typedef char false_type[2];
+
+        template <typename S>
+        static true_type& check(S* s_ptr, decltype(s_ptr->begin())* = 0);
+
+        template <typename S>
+        static false_type& check(...);
+
+        static const bool value = sizeof(check<T>(0)) == sizeof(true_type);
+    };
+
+    template <typename T>
+    struct has_begin : boost::mpl::bool_<has_begin_impl<T>::value> {};
+}
+
+template <typename Range>
+inline typename boost::range_iterator<Range>::type
+begin(Range& rng, typename disable_if<range_detail::has_begin<Range>>::type* = 0)
+{
+    return range_begin(rng);
+}
+
+template <typename Range>
+inline typename boost::range_iterator<Range const>::type
+begin(Range const& rng, typename disable_if<range_detail::has_begin<Range>>::type* = 0)
+{
+    return range_begin(rng);
+}
+
+}
+#else
+
 #ifdef BOOST_NO_FUNCTION_TEMPLATE_ORDERING
 #include <boost/range/detail/begin.hpp>
 #else
@@ -128,5 +176,7 @@
     }
 }
 
+#endif // BOOST_NO_RANGE_BASED_FOR
+
 #endif
 
